// file: $ENDPOINT/api/endpoint.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "../integral_constants.h"
#include "../signal_detector.h"
#include "../signal_detector_constants.h"

// define a format for outputing endpoint information
//
#define SIGD_OUTPUT_FORMAT "%s: %4ld beg_time = %9.3f secs   end_time = %9.3f secs\n"

// define the grow size of the endpoints buffer
//
#define SIGD_ENDPTS_GROW_SIZE 10000

// local functions
//
extern int_4 grow_arrays_cc(float_4** arrays, int_4 num_arrays,
			    int_4& cur_size, int_4 grow_size);

//-----------------------------------------------------------------------------
//
// program: endpoint
// synopsis: ednpoint [options] files
// descr: automatically locates utterances in a file
// example: endpoint sigd_0.params sigd_0.params input.raw
//
// options: none
//
// arguments:
//  channel-0.param: (input) parameter file for channel-0
//  channel-1.param: (input) parameter file for channel-1
//  signal.raw: (input) binary signal file
//
//-----------------------------------------------------------------------------
main(int argc, char **argv) {
  
  // declare an output buffer
  //
  int_4 endpts_max_size = SIGD_ENDPTS_GROW_SIZE;

  // decalre memory to hold endpoint data of the two channels separately
  //
  float_4* endpts_chan_0[2];
  float_4* endpts_chan_1[2];
  endpts_chan_0[(int_4)0] = new float_4[endpts_max_size];
  endpts_chan_0[(int_4)1] = new float_4[endpts_max_size];
  endpts_chan_1[(int_4)0] = new float_4[endpts_max_size];
  endpts_chan_1[(int_4)1] = new float_4[endpts_max_size];
  
  // declare signal-related objects
  //
  Signal_detector sig_det_ref;
  Signal_detector sig_det_near;
  
  // check if all the required file names have been specified by the user
  //
  if(argc != 4){
    printf("%s\n","usage: vad.exe <parameter file 0> <parameter file 1> <input raw file> ");
    return ISIP_FALSE;
  }
  
  // load parameters
  //
  if (sig_det_ref.load_parameters_cc((char_1*)argv[1]) == ISIP_FALSE) {
    fprintf(stdout, "%s: error reading parameters (%s)\n", argv[0], argv[1]);
    return(ISIP_ERROR);
  }

  if (sig_det_near.load_parameters_cc((char_1*)argv[2]) == ISIP_FALSE) {
    fprintf(stdout, "%s: error reading parameters (%s)\n", argv[0], argv[2]);
    return(ISIP_ERROR);
  }
  
  // open the input file
  //
  FILE* fp_in = fopen(argv[3], "r");
  if (fp_in == NULL) {
    fprintf(stdout, "%s: error opening input file (%s)\n", argv[0], argv[3]);
    return(ISIP_ERROR);
  }
  
  // get the amount of data to read per i/o call
  //
  int_4 nbytes_in = sig_det_ref.get_nbytes_to_read_cc();
  fprintf(stdout, "input buffer size = %ld bytes\n", nbytes_in);

  int_2* signal_buf = new int_2[2*nbytes_in];
  memset(signal_buf, (int_4)0, 2*nbytes_in);
  
  // declare memory for the two channels
  //
  int_2* channel_0 = new int_2[nbytes_in];
  int_2* channel_1 = new int_2[nbytes_in];
  memset(channel_0, (int_4)0, nbytes_in);
  memset(channel_1, (int_4)0, nbytes_in);
      
  
  // initialize the signal detector
  //
  if (sig_det_ref.init_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error during initialization\n", argv[0]);
    return(ISIP_ERROR);
  }

  // initialize the signal detector
  //
  if (sig_det_near.init_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error during initialization\n", argv[0]);
    return(ISIP_ERROR);
  }

  //---------------------------------------------------------------------------
  //
  // main processing loop
  //
  //---------------------------------------------------------------------------

  // loop over the data
  //
  int_4 nf = (int_4)0;
  int_4 num_utt_chan_0 = (int_4)0;
  int_4 num_utt_chan_1 = (int_4)0;
  float_4 t1_0, t2_0, t1_1, t2_1;
  
  while (fread(signal_buf,(int_2)1, 2*nbytes_in,fp_in) > (int_4)0) {
    
    // deinterleave the data into their respective channels
    // so that the two objects could work on them simultaneously
    //
    for ( int_4 kk = 0; kk < nbytes_in; kk++) {
      channel_0[kk] = signal_buf[2*kk];
      channel_1[kk] = signal_buf[2*kk+1];
    }
    
    // process data
    //
    if (sig_det_ref.process_cc((void*)channel_0) == ISIP_FALSE) {
      fprintf(stdout, "%s: error processing frame no. %ld\n", argv[0], nf);
      return(ISIP_ERROR);
    }

    if (sig_det_near.process_cc((void*)channel_1) == ISIP_FALSE) {
      fprintf(stdout, "%s: error processing frame no. %ld\n", argv[0], nf);
      return(ISIP_ERROR);
    }

    // check for an utterance in channel_0
    //
    if (sig_det_ref.get_endpoints_cc(t1_0, t2_0) == ISIP_TRUE) {
      
      // make sure we have space for the new endpts
      //
      if (num_utt_chan_0 == endpts_max_size) {
	grow_arrays_cc(endpts_chan_0, (int_4)2,
		       endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
      }
      
      // save the endpoints
      //
      (endpts_chan_0[(int_4)0])[num_utt_chan_0] = t1_0;
      (endpts_chan_0[(int_4)1])[num_utt_chan_0] = t2_0;
      
      // send some output to the display and the output file
      //
      fprintf(stdout, SIGD_OUTPUT_FORMAT, "channel_0", num_utt_chan_0,
	      (endpts_chan_0[(int_4)0])[num_utt_chan_0],
	      (endpts_chan_0[(int_4)1])[num_utt_chan_0]);
      
      // increment counters
      //
      num_utt_chan_0++;
      sig_det_ref.reset_cc();
    }

    // check for an utterance in channel_1
    //
    if (sig_det_near.get_endpoints_cc(t1_1, t2_1) == ISIP_TRUE) {

      // make sure we have space for the new endpts
      //
      if (num_utt_chan_1 == endpts_max_size) {
	grow_arrays_cc(endpts_chan_1, (int_4)2,
		       endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
      }

      // save the endpoints
      //
      (endpts_chan_1[(int_4)0])[num_utt_chan_1] = t1_1;
      (endpts_chan_1[(int_4)1])[num_utt_chan_1] = t2_1;
      
      // send some output to the display and the output file
      //
      fprintf(stdout, SIGD_OUTPUT_FORMAT, "channel_1" ,num_utt_chan_1,
	      (endpts_chan_1[(int_4)0])[num_utt_chan_1],
	      (endpts_chan_1[(int_4)1])[num_utt_chan_1]);
      
      // increment counters
      //
      num_utt_chan_1++;
      sig_det_near.reset_cc();
    }
    
    // increment counters
    //
    memset(signal_buf, (int_4)0, 2*nbytes_in);
    memset(channel_0, (int_4)0, nbytes_in);
    memset(channel_1, (int_4)0, nbytes_in);
    nf++;
  }
  
  //---------------------------------------------------------------------------
  //
  // cleanup
  //
  //---------------------------------------------------------------------------

  // 
  //
  if (sig_det_ref.flush_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error flushing signal detector\n", argv[0]);
    return(ISIP_ERROR);
  }

  if (sig_det_near.flush_cc() == ISIP_FALSE) {
    fprintf(stdout, "%s: error flushing signal detector\n", argv[0]);
    return(ISIP_ERROR);
  }
    
  // check one last time for an utterance in channel_0
  //
  if (sig_det_ref.get_endpoints_cc(t1_0, t2_0) == ISIP_TRUE) {
    
    // make sure we have space for the new endpts
    //
    if (num_utt_chan_0 == endpts_max_size) {
      grow_arrays_cc(endpts_chan_0, (int_4)2,
		     endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
    }
    
    // save the endpoints
    //
    (endpts_chan_0[(int_4)0])[num_utt_chan_0] = t1_0;
    (endpts_chan_0[(int_4)1])[num_utt_chan_0] = t2_0;

    // send some output to the display
    //
    fprintf(stdout, SIGD_OUTPUT_FORMAT, "channel_0", num_utt_chan_0, 
	    (endpts_chan_0[(int_4)0])[num_utt_chan_0],
	    (endpts_chan_0[(int_4)1])[num_utt_chan_0]);
    
    // increment counters
    //
    num_utt_chan_0++;
    sig_det_ref.reset_cc();
  }

    // check one last time for an utterance in channel_0
  //
  if (sig_det_ref.get_endpoints_cc(t1_1, t2_1) == ISIP_TRUE) {
    
    // make sure we have space for the new endpts
    //
    if (num_utt_chan_1 == endpts_max_size) {
      grow_arrays_cc(endpts_chan_1, (int_4)2,
		     endpts_max_size, SIGD_ENDPTS_GROW_SIZE);
    }
    
    // save the endpoints
    //
    (endpts_chan_1[(int_4)0])[num_utt_chan_1] = t1_1;
    (endpts_chan_1[(int_4)1])[num_utt_chan_1] = t2_1;

    // send some output to the display
    //
    fprintf(stdout, SIGD_OUTPUT_FORMAT, "channel_1", num_utt_chan_1, 
	    (endpts_chan_1[(int_4)0])[num_utt_chan_1],
	    (endpts_chan_1[(int_4)1])[num_utt_chan_1]);
    
    // increment counters
    //
    num_utt_chan_1++;
    sig_det_near.reset_cc();
  }

  //---------------------------------------------------------------------------
  //
  // exit
  //
  //---------------------------------------------------------------------------

  // close files
  //
  fclose(fp_in);

  // exit gracefully
  //
  return(ISIP_NO_ERROR);
}






